<?php
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use SimpleXMLElement;

class WebService {
    
    //const URL_SITE = 'https://hml-cashback.teknisa.com';
    
    const URL_SHORTENER = 'https://sh.teknisa.com';
    const USER_SHORTENER = 'teknisa';
    const PWD_SHORTENER = 'teknisa';
    
    private $util;
    private $integrationService;
    
    private $basePath;
    private $ordersPath;        
    private $processedPath;     
    private $errPath;
    
    private $mailList;
    private $serverName;
    
    private $useSMS = false;
    private $useQRCode = false;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Service\IntegrationService $integrationService) {
        
        $this->connection         = $connection;
        $this->util               = $util;
        $this->integrationService = $integrationService;

        $config = $this->getClientName();
        
        if(isset($config['urlSendType'])) {
            $this->useSMS = in_array($config['urlSendType'], ['ALL', 'SMS']);
            $this->useQRCode = in_array($config['urlSendType'], ['ALL', 'QRCODE']);
        }
    }
    
    public function welcome (Request $request, Response $response) {
        $this->util->writeLog('execution', '', 'init route /welcome');
        
        $sql         = "SELECT CDFILIAL FROM FILIAL";
        $result      = $this->connection->fetchAll($sql);
        $versionInfo = $this->getVersionInfo();
        
        $data = [
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => str_replace('/', '-', $versionInfo['date']),
            'branchs'     => count($result)
        ];
        
        $return = $this->returnStructure('false', '' , $data);
        
        
        $response = $response->withHeader('Content-Type', "application/xml");
        $response->getBody()->write($return);
        
        return $response;
    }
    
    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    private function getClientName() {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    public function getBalance(Request $request, Response $response) {
        $this->util->writeLog('execution', '', 'init route /getBalance');

        try {
            $params   = $request->getParsedBody();
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            if (isset($input['data']['clientCode'])) {
                $clientCode = $input['data']['clientCode'];
            } else {
                throw new Exception\ParamsException("Código de cliente não informado.");
            }
            
            if (isset($input['data']['consumerCode'])) {
                $consumerCode = $input['data']['consumerCode'];
            } else {
                throw new Exception\ParamsException("Código de consumidor não informado.");
            }
            
            if (isset($input['data']['campaignCode'])) {
                $campaignCode = $input['data']['campaignCode'];
            } else {
                throw new Exception\ParamsException("Código de campanha não informado.");
            }
            
            $balance = $this->integrationService->getBalance($clientCode, $consumerCode, $campaignCode);
            
            $return = $this->returnStructure('false', '' , $balance);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $message = 'Erro ao consultar saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function creditBalance(Request $request, Response $response) {
        $this->util->writeLog('execution', '', 'init route /creditBalance');
        global $CLIENT_NAME;
        
        $params = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
             if (isset($input['data']['subsidiaryCode']) && !empty($input['data']['subsidiaryCode'])) {
                $subsidiaryCode = $input['data']['subsidiaryCode'];
            } else {
                throw new Exception\ParamsException("Código da Filial não informado.");
            }
            
            if (isset($input['data']['clientCode'])) {
                $clientCode = $input['data']['clientCode'];
            } else {
                throw new Exception\ParamsException("Código do Cliente não informado.");
            }
            
            if (isset($input['data']['consumerCode'])) {
                $consumerCode = $input['data']['consumerCode'];
            } else {
                throw new Exception\ParamsException("Código do Consumidor não informado.");
            }
            
            if (isset($input['data']['phone']) && !empty($input['data']['phone'])) {
                $phone = preg_replace('/[\(\)\s-]/', '', $input['data']['phone']);
            } else {
                $phone = null;
            }
            
            if (isset($input['data']['posCode'])) {
                $posCode = $input['data']['posCode'];
            } else {
                throw new Exception\ParamsException("Caixa não informado.");
            }
            
            if (isset($input['data']['saleSeq'])) {
                $saleSeq = str_pad($input['data']['saleSeq'], 10, 0, STR_PAD_LEFT);
            } else {
                throw new Exception\ParamsException("saleSeq não informado.");
            }
            
            $branch = $this->integrationService->getBranch($subsidiaryCode);
            
            if(empty($branch))
                throw new Exception\ParamsException("Filial {$subsidiaryCode} não encontrada.");

            $checkRegConsumer = $this->integrationService->registerConsumer($subsidiaryCode, $clientCode, $consumerCode, $phone);
            
            $this->util->writeLog('execution', '', $checkRegConsumer['message']);

            $dtAbertur      = isset($input['data']['posOpeningDate']) ? $input['data']['posOpeningDate'] : null;
            $dataProducts   = isset($input['data']['products']) ? $input['data']['products'] : null;

            if(!isset($dataProducts['prod'][0])) {
                $products['prod'][0] = $dataProducts['prod'];
                unset($dataProducts['prod']);
                $dataProducts['prod'] = $products['prod'];
            }
            
            $score = $this->integrationService->processScore($subsidiaryCode, $posCode, $checkRegConsumer['data'], $dataProducts['prod'], $saleSeq, $dtAbertur);
        
            $data = [
                'consumerName' => $checkRegConsumer['data']['NMCONSUMIDOR'],
                'consumerCode' => $checkRegConsumer['data']['CDCONSUMIDOR'],
                'clientCode'   => $checkRegConsumer['data']['CDCLIENTE'],
                'qrCode'       => '',
                'balance'      => $score
            ];
            
            if ($data['balance'] != 0 && !empty($phone)) {
                $message = $this->formatSMS($branch, $data['clientCode'], $data['consumerCode'], $data['consumerName'], $data['balance']['generatedBalance'], $data['balance']['expiration'], 'S', $phone);
                $send = $this->sendSMS($phone, $message);
                
                if($this->useSMS && (!isset($send['Success']) || !$send['Success']))
                    $this->util->writeLog('execution', '', 'Falha no envio de SMS!');
            }
            
            $this->util->writeLog('execution', '', strtoupper($data['consumerName']));
            
            if(strtoupper($data['consumerName']) == 'NOVO CONSUMIDOR' && $this->useSMS && !empty($phone)) {
                $qrCode = '';
                $message = $this->formatSMS($branch, $data['clientCode'], $data['consumerCode'], $data['consumerName'], null, null, 'C', $checkRegConsumer['data']['NRTELECONS'], $qrCode);
                if ($this->useQRCode) {
                    $data['qrCode'] = $qrCode;
                }
                $this->util->writeLog('execution', '', $message);

                if(!empty($message))
                    $send = $this->sendSMS($phone, $message);

                $this->util->writeLog('execution', '', 'msg enviada');
            }
            
            $this->connection->commit();

            $return = $this->returnStructure('false', '' , $data);
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro ao creditar saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function redeemBalance(Request $request, Response $response) {
        $this->util->writeLog('execution', '', 'init route /redeemBalance');
        $params   = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            
            if (isset($input['data']['subsidiaryCode']) && !empty($input['data']['subsidiaryCode'])) {
                $subsidiaryCode = $input['data']['subsidiaryCode'];
            } else {
                throw new Exception\ParamsException("Código da Filial não informado.");
            }
            
            if (isset($input['data']['clientCode'])) {
                $clientCode = $input['data']['clientCode'];
            } else {
                throw new Exception\ParamsException("Código do Cliente não informado.");
            }
            
            if (isset($input['data']['consumerCode'])) {
                $consumerCode = $input['data']['consumerCode'];
            } else {
                throw new Exception\ParamsException("Código do Consumidor não informado.");
            }
            
            if (isset($input['data']['phone']) && !empty($input['data']['phone'])) {
              $phone = preg_replace('/[\(\)\s-]/', '', $input['data']['phone']);
            } else {
                $phone = null;
            }
            
            if (isset($input['data']['saleSeq'])) {
                $saleSeq = str_pad($input['data']['saleSeq'], 10, 0, STR_PAD_LEFT);
            } else {
                throw new Exception\ParamsException("saleSeq não informado.");
            }
            
            if (isset($input['data']['posCode'])) {
                $posCode = $input['data']['posCode'];
            } else {
                throw new Exception\ParamsException("Caixa não informado.");
            }
            
            if (isset($input['data']['campaignCode'])) {
                $campaignCode = $input['data']['campaignCode'];
            } else {
                throw new Exception\ParamsException("Código de campanha não informado.");
            }
            
            $branch = $this->integrationService->getBranch($subsidiaryCode);
            
            if(empty($branch))
                throw new Exception\ParamsException("Filial {$subsidiaryCode} não encontrada.");
            
            $balanceUsed = isset($input['data']['balanceUsed']) ?  $this->util->toFloat($input['data']['balanceUsed']) : 0;
            $dtAbertur   = isset($input['data']['posOpeningDate']) ? $input['data']['posOpeningDate'] : null;
            
            $consumerDetails = $this->integrationService->getConsumer($clientCode, $consumerCode);
            $balance = $this->integrationService->getBalance($clientCode, $consumerCode, $campaignCode);
            
            if(!$input['data']['balanceUsed'])
                throw new Exception\ParamsException("O valor de resgate solicitado é inválido.");
            
            if($balance['balance'] < $balanceUsed)
                throw new Exception\ParamsException("Saldo insuficiente. Saldo: {$balance['balance']} | Valor de Resgate: {$balanceUsed}");
            
            $this->integrationService->redeemBalance($subsidiaryCode, $posCode, $consumerDetails, $campaignCode, $balanceUsed, $saleSeq, $dtAbertur);
            
            $message = $this->formatSMS($branch, $consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR'], $consumerDetails['NMCONSUMIDOR'], $balanceUsed, null, 'R', $phone);
            
            if($this->useSMS && !empty($phone)) {
                $send = $this->sendSMS($phone, $message);

                if($this->useSMS && (!isset($send['Success']) || !$send['Success']))
                    $this->util->writeLog('execution', '', 'Falha no envio de SMS!');
            
            }
            
            $this->connection->commit();
            
            $return = $this->returnStructure('false', 'Resgate de saldo realizado com sucesso.' , []);
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro ao resgatar saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function cancelBalance(Request $request, Response $response) {
        $this->util->writeLog('execution', '', 'init route /cancelBalance');
        $params   = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            
           if (!isset($input['data']['subsidiaryCode']) || empty($input['data']['subsidiaryCode']))
                throw new Exception\ParamsException("Código da Filial não informado.");
                
            if (!isset($input['data']['clientCode']) || empty($input['data']['clientCode']))
                throw new Exception\ParamsException("Código do Cliente não informado.");
            
            if (!isset($input['data']['consumerCode']) || empty($input['data']['consumerCode']))
                throw new Exception\ParamsException("Código do consumerCode não informado.");
                
            if (!isset($input['data']['saleSeq']) || empty($input['data']['saleSeq']))
                throw new Exception\ParamsException("sequencial da venda não informado.");
                
            if (!isset($input['data']['posOpeningDate']) || empty($input['data']['posOpeningDate']))
                throw new Exception\ParamsException("Data de abertura do caixa não informado.");
                
            if (!isset($input['data']['posCode']) || empty($input['data']['posCode']))
                throw new Exception\ParamsException("Caixa não informado.");
            
            
            $subsidiaryCode = $input['data']['subsidiaryCode'];
            $clientCode     = $input['data']['clientCode'];
            $consumerCode   = $input['data']['consumerCode'];
            $saleSeq        = str_pad($input['data']['saleSeq'], 10, 0, STR_PAD_LEFT);
            $posCode        = $input['data']['posCode'];
            $dtabertur      = $input['data']['posOpeningDate'];
            
            $branch = $this->integrationService->getBranch($subsidiaryCode);
            
            if(empty($branch))
                throw new Exception\ParamsException("Filial {$subsidiaryCode} não encontrada.");
            
            $consumerDetails = $this->integrationService->getConsumer($clientCode, $consumerCode);

            $this->integrationService->cancelBalance($subsidiaryCode, $posCode, $consumerDetails, $saleSeq, $dtabertur);
            
            $return = $this->returnStructure('false', 'Cancelamento realizado com sucesso.' , []);
            
            $this->connection->commit();
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro ao cancelar credito/resgate de saldo. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function getSaleResume(Request $request, Response $response) {
    
        $this->util->writeLog('execution', '', 'init route /getSaleResume');
        $params   = $request->getParsedBody();
        
        try {
            
           if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            
            if (!isset($input['data']['subsidiaryCode']) || empty($input['data']['subsidiaryCode']))
                throw new Exception\ParamsException("Código da Filial não informado.");
                
            if (!isset($input['data']['clientCode']) || empty($input['data']['clientCode']))
                throw new Exception\ParamsException("Código do Cliente não informado.");
            
            if (!isset($input['data']['consumerCode']) || empty($input['data']['consumerCode']))
                throw new Exception\ParamsException("Código do consumerCode não informado.");
                
            if (!isset($input['data']['saleSeq']) || empty($input['data']['saleSeq']))
                throw new Exception\ParamsException("sequencial da venda não informado.");
                
            if (!isset($input['data']['posOpeningDate']) || empty($input['data']['posOpeningDate']))
                throw new Exception\ParamsException("Data de abertura do caixa não informado.");
                
            if (!isset($input['data']['posCode']) || empty($input['data']['posCode']))
                throw new Exception\ParamsException("Caixa não informado.");

            
            $subsidiaryCode = $input['data']['subsidiaryCode'];
            $saleSeq        = str_pad($input['data']['saleSeq'], 10, 0, STR_PAD_LEFT);
            $posCode        = $input['data']['posCode'];
            $dtabertur      = $input['data']['posOpeningDate'];
            
            $consumerDetails['CDCLIENTE']    = $input['data']['clientCode'];
            $consumerDetails['CDCONSUMIDOR'] = $input['data']['consumerCode'];
            $saleResume = $this->integrationService->getMovementConsumer($subsidiaryCode, $posCode, $consumerDetails, $saleSeq, $dtabertur);

            $credit = 0;
            $debit  = 0;
            $expiration = null;
            $campaignCode = null;
            
            if(empty($saleResume))
                throw new Exception\ParamsException("Venda não encontrada. Verifique os parametros enviados");
                
            foreach($saleResume as $movement) {

                if($movement['IDCAMPANHA'] == 'S') {

                    $campaignCode = $movement['CDCAMPANHA'];

                    if($movement['IDTPMOVCONS'] == 'C') {
                        $credit = bcadd($credit, $this->util->toFloat($movement['VRMOVPONTOCONS']), 2);
                        $expiration = str_replace('-', '/', $movement['DTVALIDAPONTO']);
                    }
                    
                    if($movement['IDTPMOVCONS'] == 'D') {
                        $debit = bcadd($debit, $this->util->toFloat($movement['VRMOVPONTOCONS']), 2);
                    }
                }
            }
            
            $consumerDetails = $this->integrationService->getConsumer($consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR']);
            $balance = $this->integrationService->getBalance($consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR'], $campaignCode);
            
            $score = [
                'generatedBalance' => $this->util->toFloat($credit),
                'redeemBalance'    => $this->util->toFloat($debit),
                'totalBalance'     => $this->util->toFloat($balance['balance']),
                'expiration'       => $expiration
            ];

            $data = [
                'consumerName' => $consumerDetails['NMCONSUMIDOR'],
                'consumerCode' => $consumerDetails['CDCONSUMIDOR'],
                'clientCode'   => $consumerDetails['CDCLIENTE'],
                'balance'      => $score
            ];

            $return = $this->returnStructure('false', '' , $data);
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $message = 'Erro. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function getConsumer(Request $request, Response $response) {
        $this->util->writeLog('execution', '', 'init route /getConsumer');
      
        $params = $request->getParsedBody();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            if (!isset($input['data']['clientCode']) || empty($input['data']['clientCode'])) {
                throw new Exception\ParamsException("Campo clientCode é não informado.");
            }
            
            if (!isset($input['data']['consumerCode']) || empty($input['data']['consumerCode'])) {
                throw new Exception\ParamsException("Campo consumerCode não informado.");
            }
            
            $consumerDetails = $this->integrationService->getConsumer($input['data']['clientCode'], $input['data']['consumerCode']);
            
            if(empty($consumerDetails))
                throw new Exception\ParamsException("Consumidor não encontrado.");
                
            if(strtoupper($consumerDetails['NMCONSUMIDOR']) != 'NOVO CONSUMIDOR') 
                throw new Exception\ParamsException("Consumidor já atualizado.");
                
            $dtnasc = null;
            if(!empty($consumerDetails['DTNASCCONS'])) {
                $consumerDetails['DTNASCCONS'] = \DateTime::createFromFormat('d/m/y', $consumerDetails['DTNASCCONS']);
                $dtnasc = $consumerDetails['DTNASCCONS']->format('d/m/Y');
            }
            
            $gender = null;
            if(!empty($consumerDetails['IDSEXOCONS'])) {
                $gender = strtoupper($consumerDetails['IDSEXOCONS']) == 'F' ? 'Feminino' : 'Masculino'; 
            }

            $data = [
                'clientCode'    => $consumerDetails['CDCLIENTE'],
                'consumerCode'  => $consumerDetails['CDCONSUMIDOR'],
                'name'          => null,
                'email'         => $consumerDetails['DSEMAILCONS'],
                'phone'         => $consumerDetails['NRTELECONS'],
                'cpf'           => $consumerDetails['NRCPFRESPCON'],
                'birthday'      => $dtnasc,
                'gender'        => $gender,
                'cep'           => $consumerDetails['NRCEPCONS'],
                'street'        => $consumerDetails['DSENDECONS'],
                'neighborhood'  => $consumerDetails['NMBAIRCONS'],
                'addressNumber' => $consumerDetails['NRENDECONS'],
                'complement'    => $consumerDetails['DSCOMPLENDECONS'],
                'state'         => $consumerDetails['SGESTADO'],
                'city'          => $consumerDetails['NMMUNICIPIO']
            ];
            
            $return = $this->returnStructure('false', '' , $data);
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $message = 'Erro. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);
        
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function updateConsumer(Request $request, Response $response) {
        $this->util->writeLog('execution', '', '/updateConsumer');
        $params = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            
            if (!isset($input['data']['clientCode']) || empty($input['data']['clientCode'])) {
                throw new Exception\ParamsException("Codigo de cliente não informado.");
            }
            
            if (!isset($input['data']['consumerCode']) || empty($input['data']['consumerCode'])) {
                throw new Exception\ParamsException("Codigo de consumidor não informado.");
            }
            
            $this->integrationService->updateConsumer($input['data']);
            
            $return = $this->returnStructure('false', 'Registro atualizado com sucesso.' , []);
            
            $this->connection->commit();
            
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro na atualização de dados. ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);

            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }

    private function returnStructure($status, $message, $data) {
        $array = [
            'error'   => $status,
            'message' => $message,
            'data'    => $data
        ];

        $return = $this->buildXmlReturn($array);
        return $return;
    }
    

    private function buildXmlReturn($array) {
        $xml  = new \DOMDocument("1.0", "UTF-8");
        $root = $xml->createElement('root');
      
        $xml->appendChild($root);
        
        function add_element($xml, $parent, $key, $value) {
          $entities = [
              ">"  => "&gt;",
              "<"  => "&lt;",
              "&"  => "&amp;",
              "\"" => "&quot;",
              "'"  => "&apos;"
            ];
          
          if (is_array($value)) {
            $element = $xml->createElement(str_replace(array_keys($entities), array_values($entities), $key));
            $parent->appendChild($element);
            foreach ($value as $subkey => $subvalue) {
                add_element($xml, $element, $subkey, $subvalue);
            }
          } else {
            if (empty($value) && !is_numeric($value)) {
                $element = $xml->createElement($key);
                $element->appendChild($xml->createTextNode(""));
            } else {
                $element = $xml->createElement(str_replace(array_keys($entities), array_values($entities), $key), str_replace(array_keys($entities), array_values($entities), $value));
            }
            $parent->appendChild($element);
          }
        }
        foreach ($array as $key => $value) {
            add_element($xml, $root, $key, $value);
        }
        
        $xml->formatOutput = true;
        return $xml->saveXML();
    }
    
    private function formatSMS($branch,$clientCode, $consumerCode, $consumerName, $value = 0, $expiration = null, $type, $phone, &$qrCode = '') {
        $consumerName = ucwords(strtolower(substr($consumerName, 0, 15)));
        $parts = explode(' ', strtolower($consumerName));
        $firstName = ucfirst($parts[0]);
        
        $consumerName = preg_replace('/[^\p{L}\p{N}\s]/u', '', $firstName);
        //$branchName  = substr($branch['NMFILIAL'], 0, 20);
        
        $today = date('d/m');
        $value = bcmul($value, '1', 2);

        if(!empty($expiration))
            $expiration = date('d/m', strtotime(str_replace("/", "-", $expiration)));
        
        if($type == 'S') {

            $partMsg1 = "GIRAFFAS: ";
            $partMsg2 = "Parabens, {$consumerName}! ";
            $partMsg3 = "Voce ganhou R$" . str_replace('.', ',', $value) . " de cashback para sua proxima compra, aproveite.";
            $partMsg4 = " Valido de {$today} a {$expiration}.";
            
            if($firstName == 'Novo') {
                $finalMessage = $partMsg1 . $partMsg3;
            } else {
                $finalMessage = $partMsg1 . $partMsg2 . $partMsg3;
            }
            
            $finalMessage = !empty($expiration) ? $finalMessage . $partMsg4 : $finalMessage;
        }
        
        if($type == 'R') {
            
            $partMsg1 = "GIRAFFAS: ";
            $partMsg2 = "Ola {$consumerName}! ";
            $partMsg3 = "Seu resgate de R$" . str_replace('.', ',', $value) ." de cashback foi concluido com sucesso. Esperamos te ver em breve.";
            
            if($firstName == 'Novo') {
                $finalMessage = $partMsg1 . $partMsg3;
            } else {
                $finalMessage = $partMsg1 . $partMsg2 . $partMsg3;
            }
        }
        
        if($type == 'C') {
            if(empty($phone))
              throw new Exception\ParamsException("Telefone do consumidor inválido ou não informado.");

            $config  = $this->getClientName();
            
             $params = [
                'clientCode'   => $clientCode,
                'consumerCode' => $consumerCode
            ];
            
            $this->util->writeLog('execution', '', "UrlBase: {$config['urlSite']}");
            $urlSite = $config['urlSite'] . "/getConfig?id=" . base64_encode(json_encode($params));

            
            //$urlSite = $config['urlSite'] . "/getConfig?phone={$phone}";
            
            
            $auth = base64_encode(self::USER_SHORTENER . ':' . self::PWD_SHORTENER);
            $header = [
                "Authorization: Basic {$auth}",
                "Content-Type: application/json"
            ];
            $body = ["url" => $urlSite];

            $curl = curl_init();
            curl_setopt_array($curl, array(
              CURLOPT_URL               => self::URL_SHORTENER .'/shorten',
              CURLOPT_RETURNTRANSFER    => true,
              CURLOPT_TIMEOUT           => 4,
              CURLOPT_HTTP_VERSION      => CURL_HTTP_VERSION_1_1,
              CURLOPT_SSL_VERIFYPEER    => false,
              CURLOPT_CUSTOMREQUEST     => 'POST',
              CURLOPT_POSTFIELDS        => json_encode($body),
              CURLOPT_HTTPHEADER        => $header,
            ));
            
            $curlResponse = curl_exec($curl);
            $httpCode     = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);
            
            $rUrl = json_decode($curlResponse, true);
            
            if($httpCode != 200 || !isset($rUrl['shortUrl'])) {
                $this->util->writeLog('execution', '', 'Falha ao gerar link de atualização de cadastro.');
                return null;
            }
            
            $shortenedLink = $rUrl['shortUrl'];
            $qrCode = isset($rUrl['qrCode']) ? $rUrl['qrCode'] : '';
            
            $partMsg1 = "GIRAFFAS: ";
            $partMsg2 = "Ola {$consumerName}! ";
            $partMsg3 = "Complete seu cadastro agora e desfrute das vantagens exclusivas. Acesse: ". $shortenedLink;
            
            if($firstName == 'Novo') {
                $finalMessage = $partMsg1 . $partMsg3;
            } else {
                $finalMessage = $partMsg1 . $partMsg2 . $partMsg3;
            }
        }

        return $finalMessage;
    }
    
    private function sendSMS($phone, $message) {
        if (!$this->useSMS) return [];

        $config  = $this->getClientName(); 
        
        $header = [
            "auth-key: {$config['tokenApi']}",
            "Content-Type: application/json"
        ];
        
        $body = [
            "Receivers" => $phone,
            "Content"   => $message
        ];
        
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL               => $config['urlApiSMS'].'/send',
          CURLOPT_RETURNTRANSFER    => true,
          CURLOPT_ENCODING          => '',
          CURLOPT_MAXREDIRS         => 10,
          CURLOPT_TIMEOUT           => 4,
          CURLOPT_HTTP_VERSION      => CURL_HTTP_VERSION_1_1,
          CURLOPT_SSL_VERIFYPEER    => false,
          CURLOPT_CUSTOMREQUEST     => 'POST',
          CURLOPT_POSTFIELDS        => json_encode($body),
          CURLOPT_HTTPHEADER        => $header,
        ));
        
        $curlResponse = curl_exec($curl);
        $httpCode     = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($curl)) || ($httpCode != 200) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
            $fileName = "REQ-".date("H-i-s") . ".txt";

            $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $requestHeader,
                'url'       => $url,
                'method'    => $method,
                'params'    => $requestBody,
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curlResponse
            ]));
        }
        
        curl_close($curl);
        
        return json_decode($curlResponse, true);
        
    }
    
    public function generateQrcode(Request $request, Response $response) {
        $this->util->writeLog('execution', '', 'init route /generateQrcode');
        $params   = $request->getParsedBody();
        
        $this->connection->beginTransaction();
        
        try {
            
            if (empty($params)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }

            $xml = simplexml_load_string($params->asXML());
            
            if ($xml === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $input = json_decode(json_encode($xml), true);
            
            if (!isset($input['data']['subsidiaryCode']) && empty($input['data']['subsidiaryCode']))
                 throw new Exception\ParamsException("Código da Filial não informado.");
                 
            if (!isset($input['data']['clientCode']) && empty($input['data']['clientCode']))
                 throw new Exception\ParamsException("Código do Cliente não informado.");
                 
            if (!isset($input['data']['consumerCode']) && empty($input['data']['consumerCode']))
                 throw new Exception\ParamsException("Código do Consumidor não informado.");
                 

            $subsidiaryCode = $input['data']['subsidiaryCode'];
            $clientCode     = $input['data']['clientCode'];
            $consumerCode   = $input['data']['consumerCode'];
            $phone          = $input['data']['phone'];
            
            if (empty($this->integrationService->getBranch($subsidiaryCode)))
                 throw new Exception\ParamsException("Filial {$subsidiaryCode} não encontrada.");
                 
            $checkRegConsumer = $this->integrationService->registerConsumer($subsidiaryCode, $clientCode, $consumerCode, $phone);
            
            if($checkRegConsumer['data']['IDCADCONSATU'] != 'S') {
            
                $config  = $this->getClientName(); 
                
                $params = [
                    'clientCode'   => $clientCode,
                    'consumerCode' => $consumerCode
                ];
                
                $urlSite = $config['urlSite'] . "/getConfig?id=" . base64_encode(json_encode($params));
    
                $auth = base64_encode(self::USER_SHORTENER . ':' . self::PWD_SHORTENER);
                $header = [
                    "Authorization: Basic {$auth}",
                    "Content-Type: application/json"
                ];
                $body = ["url" => $urlSite];
    
                $curl = curl_init();
                curl_setopt_array($curl, array(
                  CURLOPT_URL               => self::URL_SHORTENER .'/shorten',
                  CURLOPT_RETURNTRANSFER    => true,
                  CURLOPT_TIMEOUT           => 4,
                  CURLOPT_HTTP_VERSION      => CURL_HTTP_VERSION_1_1,
                  CURLOPT_SSL_VERIFYPEER    => false,
                  CURLOPT_CUSTOMREQUEST     => 'POST',
                  CURLOPT_POSTFIELDS        => json_encode($body),
                  CURLOPT_HTTPHEADER        => $header,
                ));
                
                $curlResponse = curl_exec($curl);
                $httpCode     = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                curl_close($curl);
                
                $rUrl = json_decode($curlResponse, true);
                
                if($httpCode != 200 || !isset($rUrl['shortUrl'])) {
                    $this->util->writeLog('execution', '', 'Falha ao gerar link de atualização de cadastro.');
                    return null;
                }
                
                $return = [
                    'clientCode'   => $clientCode,
                    'consumerCode' => $consumerCode,
                    'url'          => $rUrl['shortUrl']
                ];
                
            } else {
                 $return = [
                    'clientCode'   => $clientCode,
                    'consumerCode' => $consumerCode,
                    'url'          => ''
                ];
            }
            
            $return = $this->returnStructure('false', '' , $return);
            
            $this->connection->commit();
    
            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            
            return $response;
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            $message = 'Erro ao gerar QrCode ' . $e->getMessage();
            $return = $this->returnStructure('true', $message , null);
            $this->util->writeLog('execution', '', $message);

            $response = $response->withHeader('Content-Type', "application/xml");
            $response->getBody()->write($return);
            return $response->withStatus(400);
        }
    }
    
    public function debug(Request $request, Response $response) {
        $phone = '31986834520';
        $message = $this->formatSMS('teste', 'teste', 0, null, 'C', $phone);

        $send = $this->sendSMS($phone, $message);
        
        $return = $this->returnStructure('false', '' , $send);

        $response = $response->withHeader('Content-Type', "application/xml");
        $response->getBody()->write($return);
        
        return $response;
    }
}
